use crate::types::{InputCause, ButtonState, ProcessingResult};

// ------------------- InputCause ------------------- //

#[test]
fn input_cause_is_engaged() {
    assert!(InputCause::Button(ButtonState::Pressed).is_engaged() == true);
    assert!(InputCause::Button(ButtonState::Released).is_engaged() == false);
    assert!(InputCause::Axis(0, 0.0).is_engaged() == false);
    assert!(InputCause::Axis(1, 0.0).is_engaged() == false);
    assert!(InputCause::Axis(1, 0.1).is_engaged() == true);
    assert!(InputCause::Motion(0.0, 0.0).is_engaged() == false);
    assert!(InputCause::Motion(0.0, 1.0).is_engaged() == false);
    assert!(InputCause::Motion(1.0, 1.0).is_engaged() == false);
}

#[test]
fn input_cause_is_axis() {
    assert!(InputCause::Button(ButtonState::Released).is_axis() == false);
    assert!(InputCause::Axis(0, 0.0).is_axis() == true);
    assert!(InputCause::Motion(0.0, 0.0).is_axis() == false);
}

#[test]
fn input_cause_scalar_value() {
    assert!(InputCause::Button(ButtonState::Pressed).scalar_value() == 1.0);
    assert!(InputCause::Button(ButtonState::Released).scalar_value() == 0.0);
    assert!(InputCause::Axis(0, 0.0).scalar_value() == 0.0);
    assert!(InputCause::Axis(1, 0.0).scalar_value() == 0.0);
    assert!(InputCause::Axis(1, 0.1).scalar_value() == 0.1);
    assert!(InputCause::Motion(0.0, 0.0).scalar_value() == 0.0);
    assert!(InputCause::Motion(0.0, 1.0).scalar_value() == 0.0);
    assert!(InputCause::Motion(1.0, 1.0).scalar_value() == 1.0);
}

// ------------------- ProcessingResult ------------------- //

#[test]
fn processing_result_input_is_engaged() {
    let pr_with_cause = |cause| {
        ProcessingResult {
            action: 0,
            player_id: Some(0),
            cause: Some(cause),
        }
    };

    assert!(pr_with_cause(InputCause::Button(ButtonState::Pressed)).input_is_engaged() == true);
    assert!(pr_with_cause(InputCause::Button(ButtonState::Released)).input_is_engaged() == false);
    assert!(pr_with_cause(InputCause::Axis(0, 0.0)).input_is_engaged() == false);
    assert!(pr_with_cause(InputCause::Axis(1, 0.0)).input_is_engaged() == false);
    assert!(pr_with_cause(InputCause::Axis(1, 0.1)).input_is_engaged() == true);
    assert!(pr_with_cause(InputCause::Motion(0.0, 0.0)).input_is_engaged() == false);
    assert!(pr_with_cause(InputCause::Motion(0.0, 1.0)).input_is_engaged() == false);
    assert!(pr_with_cause(InputCause::Motion(1.0, 1.0)).input_is_engaged() == false);

    assert!(ProcessingResult { action: 0, player_id: Some(0), cause: None }.input_is_engaged() == false);
}

#[test]
fn processing_result_input_is_axis() {
    let pr_with_cause = |cause| {
        ProcessingResult {
            action: 0,
            player_id: Some(0),
            cause: Some(cause),
        }
    };

    assert!(pr_with_cause(InputCause::Button(ButtonState::Pressed)).input_is_axis() == false);
    assert!(pr_with_cause(InputCause::Axis(0, 0.0)).input_is_axis() == true);
    assert!(pr_with_cause(InputCause::Motion(0.0, 0.0)).input_is_axis() == false);


    assert!(ProcessingResult { action: 0, player_id: Some(0), cause: None }.input_is_axis() == false);
}

#[test]
fn processing_result_input_scalar_value() {
    let pr_with_cause = |cause| {
        ProcessingResult {
            action: 0,
            player_id: Some(0),
            cause: Some(cause),
        }
    };

    assert!(pr_with_cause(InputCause::Button(ButtonState::Pressed)).input_scalar_value() == 1.0);
    assert!(pr_with_cause(InputCause::Button(ButtonState::Released)).input_scalar_value() == 0.0);
    assert!(pr_with_cause(InputCause::Axis(0, 0.0)).input_scalar_value() == 0.0);
    assert!(pr_with_cause(InputCause::Axis(1, 0.0)).input_scalar_value() == 0.0);
    assert!(pr_with_cause(InputCause::Axis(1, 0.1)).input_scalar_value() == 0.1);
    assert!(pr_with_cause(InputCause::Motion(0.0, 0.0)).input_scalar_value() == 0.0);
    assert!(pr_with_cause(InputCause::Motion(0.0, 1.0)).input_scalar_value() == 0.0);
    assert!(pr_with_cause(InputCause::Motion(1.0, 1.0)).input_scalar_value() == 1.0);

    assert!(ProcessingResult { action: 0, player_id: Some(0), cause: None }.input_scalar_value() == 0.0);
}